

(eval-when (:compile-toplevel :load-toplevel :execute)
  (asdf:oos 'asdf:load-op :fiveam)
  (asdf:oos 'asdf:load-op :utilities)
  (asdf:oos 'asdf:load-op :utilities-testing))


(in-package :utilities-testing)

(def-suite utilities-suite :description "Test suite with some utilities.")
(in-suite utilities-suite)


(progn  ; a fixture would be better...
  (defparameter fn1 #'numberp)
  (defparameter inv-fn1 nil)
  (defparameter key1 #'car)
  (defparameter test1 #'equalp)
  (defparameter filter-set1 nil)

  (defmacro loopfn (f)
    (once-only-calls (f)
      `(dotimes (y 4)
	,(f 'y)))))
  

(test times 
  (is (eql (let ((x 0)) (times 10 (incf x)))
	   (dotimes (x 10 x))))

  (is (eql nil (let ((x 0)) (times 0 (incf x)))))
  
  (signals negative-limit (times -1 (write "a"))))


(test once-only-calls ;; more tests with filter
  (is (equal (let ((f  #'identity))
	       (once-only-calls (f)
		 (f 1)))
	     '(let ()
	       1)))

  (is (equal (let ((f '#'identity))
	       (once-only-calls (f)
		 (f 1)))
	     '(let ()
	       1)))

  (is (equal (let ((f '#'oddp))
	       (once-only-calls (f)
		 (f 1)))
	     '(let ()
	       (oddp 1))))

  (is (= 6 (let ((x 0))
	     (labels ((f (y)
			(incf x y)))
	       (loopfn 'f)
	     x))))

  (is (= 6 (let ((x 0))
	     (labels ((f (y)
			(incf x y)))
	       (loopfn #'f)
	     x))))

  (is (= 1 (let ((x 0))
	     (labels ((f ()
			(incf x)
			#'oddp))
	       (loopfn (f))
	       x))))

  (is (= 1 (let ((x 0))
	     (labels ((f ()
			(incf x)
			#'oddp))
	       (symbol-macrolet ((g (f)))
		   (loopfn g)
		 x))))))


(test filter-if
  (is (equal '(t t t) (filter-if #'numberp '(1 a 2 b 3 c))))
  (is (equal '(t t t) (filter-if fn1 '(1 a 2 b 3 c))))
  (is (eq    nil      (filter-if #'numberp '(a b c))))
  (is (equal '(t t t) (filter-if #'numberp '((1) (a) (2) (b) (3) (c)) :key #'car)))
  (is (equal '(1 2 3) (filter-if (lambda(i) 
				   (when (numberp i) i)) 
				 '(1 a 2 b 3 c))))
  (is (eq   nil	      (filter-if (lambda(i) 
				   (when (numberp i) i)) 
 					 '(a b c))))
  (is (equal '(1 2 3) (filter-if (lambda(i) 
				   (when (numberp i) i)) 
				 '((1) (a) (2) (b) (3) (c)) :key #'car)))

  (is (equal '(t t t) (filter-if #'numberp '((1) (a) (2) (b) (3) (c)) :key key1)))
  (is (equal '(t t t) (filter-if fn1 '(1 a 2 b 3 c))))
  (is (equal '(t t t) (filter-if fn1 '((1) (a) (2) (b) (3) (c)) :key key1)))
  
  (is (eq    nil      (filter-if nil nil)))
  (is (eq    nil      (filter-if #'numberp nil)))
  (signals undefined-function (filter-if nil '(1 2)))
  (signals undefined-function (filter-if inv-fn1 '(1 2))))


(test filter
  (is (equal '(3)	    (filter 3 '(1 2 3 4 5 6))))
  (is (eq    nil	    (filter 3 '(1 2 4 5 6))))
  (is (equal '(3)	    (filter 3 '(1 2 3 4 5 6) :test #'=)))
  (is (equal '(3 3)	    (filter 3 '(1 2 3 4 3 5 6) :test #'=)))
  (is (equal '((c 3))	    (filter 3 '((a 1) (b 2) (c 3) (d 4)) :test #'= :key #'cadr)))

  (is (equal '("ac")	    (filter "ac" '("ab" "ac"  "bc" "aa"))))
  (is (equal '((2 "ac"))    (filter "ac" '((1 "ab") (2 "ac")  (3 "bc") (4 "aa"))
				    :test #'string= :key #'cadr)))
  
  (is (equal '((3))	    (filter 3 '((1) (2) (3) (4)) :test test1 :key key1)))

  (is (equal '((1 3))	    (filter '(1 3) '(1 2 (1 3) 4 5 6) :test #'equal)))
  (is (eq    nil	    (filter '(1 3) '(1 2 3 4 5 6) :test #'equal)))
  (is (eq     nil	    (filter '(1 3) '((1 a) (2 b) (3 c)) :key #'car :test #'equal)))
  (is (equal '((c (1 3)))   (filter '(1 3) '((a 1) (b 2) (c (1 3)) (d 4) (e 5)) 
				    :test #'equal :key #'cadr)))

  (is (eq   nil		    (filter nil nil)))
  (is (eq   nil		    (filter nil '(1 2 3 nil))))
  (is (eq   nil		    (filter 3 nil))))


(test filter-set
  (is (equal '(1 3)	 (filter '(1 3) '(1 2 3 4 5 6) :filter-set t)))
  (is (equal '(1)	 (filter '(1 3) '(1 2 (1 3) 4 5 6) :filter-set t)))
  (is (eq    nil	 (filter '(1 3) '(2 (1 3) 4 5 6) :filter-set t)))
  (is (equal '(1 3)	 (filter '((1) (3)) '(1 3 4 5 6) :filter-set t :filter-key #'car)))
  (is (equal '((1) (3))	 (filter '((1) (3)) '((1) (3) (4) (5) (6)) :key #'car 
				 :filter-set t :filter-key #'car)))
  (is (equal '((1) (3))	 (filter '((1) (3)) '((1) (3) (4) (5) (6)) :key #'car :test #'=
				 :filter-set t :filter-key #'car)))
  (is (equal '((1) (3))	 (filter '((a (1)) (b (3))) '((1) (3) (4) (5) (6)) :test #'equal
				 :filter-set t :filter-key #'cadr)))

  (is (equal '((1) (3))	 (filter '((1) (3)) '((1) (3) (4) (5) (6)) :key key1 :test test1
				 :filter-set t :filter-key #'car)))

  (is (equal '((1 3))	 (filter '(1 3) '(1 2 (1 3) 4 5 6) :filter-set filter-set1)))

  (is (eq   nil	    (filter nil nil :filter-set t)))
  (is (eq   nil	    (filter nil '(1 2 3 nil) :filter-set t)))
  (is (eq   nil	    (filter '(1 2) nil :filter-set t)))

  (signals type-error	 (filter "a" '(1 2 3) :filter-set t)))


;;;
;;; This is actually to test once-only-calls using filter as a test case

(test multiple-eval  
  (let ((x 0))
    (labels ((return-fn! ()
	       (incf x)
	       #'numberp)
	     (return-key! ()
	       (incf x)
	       #'identity)
	     (return-test! ()
	       (incf x)
	       #'equalp))
      (symbol-macrolet ((item (incf x))
			(key (return-key!)))
	(is (equal (+ x 1) (progn (filter-if (return-fn!) '(1 a 2 b 3 c)) 
				  x)))
	(is (equal (+ x 2) (progn (filter-if (return-fn!) '(1 a 2 b 3 c) :key (return-key!)) 
				  x)))
	(is (equal (+ x 1) (progn (filter 1 '(1 a 2 b 3 c) :key (return-key!)) 
				  x)))
	(is (equal (+ x 2) (progn (filter 1 '(1 a 2 b 3 c) :key (return-key!) :test (return-test!))
				  x)))
	(is (equal (+ x 3) (progn (filter '(1) '(1 a 2 b 3 c) :key (return-key!) 
					  :test (return-test!) :filter-set t 
					  :filter-key (return-key!))
				  x)))
	(is (equal (+ x 2) (progn (filter-if (return-fn!) '(1 a 2 b 3 c)) :key key
				  x)))
	(is (equal (+ x 1) (progn (filter item '(1 a 2 b 3 c))
				  x)))
 	(is (equal (+ x 3) (progn (filter item '(1 a 2 b 3 c) 
					  :key (return-key!) :test (return-test!))
 				  x)))))))


(run! 'utilities-suite)

(switch-compiler-optimizations 'filter :off)

(run! 'utilities-suite)

(switch-compiler-optimizations 'filter :on)



