

(in-package :utilities)


;;;
;;; Times

(define-condition negative-limit (error)
  ((where :initarg :where :initform "unknown location" :accessor where))
  (:report (lambda (c stream)
	     (format stream "Negative limit given to '~a'." 
		     (slot-value c 'where)))))

    
(defmacro times (count-expr &body body)
  "Like the Ruby idiom: 'n.times {do_something}'. Ex: '(times n (write 1) (write 2))' 
  prints N times 12 and returns the value of the last body expression, in this case, 2.
  Simple, type-saving and handy...and nice to tease Ruby fanatics."
  (once-only (count-expr)
    (with-unique-names (limit count res)
      `(if (< ,count-expr 0)
	(error 'negative-limit :where 'times)
	(do ((,res nil)
	     (,limit ,count-expr)
	     (,count 0 (1+ ,count)))
	    ((= ,count ,limit) ,res)
	  (setf ,res (progn ,@body)))))))




